/* BabelCheck */

/*  Adam Richardson, 2008 */

/* BabelCheck is in the public domain. You may alter, distribute and use it in any way you please. */

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <syslog.h>
#include "DeskLib:File.h"
#include "DeskLib:MsgTrans.h"
#include "DeskLib:Core.h"

#define BCVERSION "1.0"

#define TOKEN_LENGTH 61
#define VALUE_LENGTH 450
#define LINE_LENGTH 512
#define PATH_LENGTH 512

typedef enum
{
  sw_master,
  sw_translated,
  sw_output,
  sw_verbose,
  sw_version
} switch_nums;

int main(int argc, char *argv[])
{
  int i = 0, missingtokens = 0, longindex = 0, required = 0;
  char values[sw_output + 1][PATH_LENGTH];
  BOOL verbose = FALSE, version = FALSE;
  FILE *fpm, *fpo;
  char line[LINE_LENGTH] = "\0", rawline[LINE_LENGTH] = "\0", token[TOKEN_LENGTH] = "\0", outputvalue[VALUE_LENGTH] = "\0";
  char *ptr;
  msgtrans_filedesc *msgsptr;
  os_error *error;
  struct option long_opts[] =
  {
    {"master",     1, 0, 'm'},
    {"translated", 1, 0, 't'},
    {"output",     1, 0, 'o'},
    {"verbose",    0, 0, 'v'},
    {"help",       0, 0, 'h'},
    {0,            0, 0, 0}
  };

  /* Ensure array is empty */
  for (i = 0; i < (sw_output + 1); i++) values[i][0] = '\0';
  i = 0;
  while (i != -1)
  {
    i = getopt_long(argc, argv, "m:t:o:vh", long_opts, &longindex);
    switch (i)
    {
    case 'm':
      strncpy(values[sw_master], optarg, PATH_LENGTH-1);
      values[sw_master][PATH_LENGTH-1]='\0';
      required++;
      break;
    case 't':
      strncpy(values[sw_translated], optarg, PATH_LENGTH-1);
      values[sw_translated][PATH_LENGTH-1]='\0';
      required++;
      break;
    case 'o':
      strncpy(values[sw_output], optarg, PATH_LENGTH-1);
      values[sw_output][PATH_LENGTH-1]='\0';
      required++;
      break;
    case 'v':
      verbose = TRUE;
      break;
    case 'h':
      version = TRUE;
      break;
    default:
      break;
    }
  }
  /* Display help if there's a syntax error or a version flag passed */
  if ((required != 3) || version)
  {
    if (version)
    {
      printf("\nBabelCheck %s\n========== ===\n\n 2008 Adam Richardson\nThis program is Public Domain. You may use, alter and redistribute it as you please.\n\n", BCVERSION);
      printf("BabelCheck is a simple utility to help cross-check a translated Messages file with the original.\nIt checks that all the tokens in the original file are present in the translated file and, if any\nare missing, it inserts the original (untranslated) values into the final output file.\nTo use BabelCheck, you should pass it the original 'master' messages file and the translation.\nIt will output a third file which will have any missing tokens replaced. If any tokens are missing\nthey will be indicated on screen.\n\n");
    }
    else printf("Syntax error...\n");
    printf("Syntax: BabelCheck -m|--master <filename> -t|--translated <filename> -o|--output <filename>\n");
    if (!version)
      printf("Use 'BabelCheck -h' for full instructions.\n\n");
    else
    {
      printf("Optional switches: -v|--verbose, -h|--help\n\n");
      printf("Switches:\n--------\nmaster:     This is the filename of the original messages file. Each token in the file is looked up\n            in the 'translated' file.\n");
      printf("translated: This is the filename of an existing translation of the 'master' file. All of the tokens\n            in the 'master' file are searched for in this file.\n");
      printf("output:      The final output filename. This will either be a copy of the 'translated' file (if all\n            the tokens were found) or a merge of the 'translated' and 'output' files (if any tokens\n            were missing from the translation).\n");
      printf("verbose:    Use this switch to display extra information while BabelCheck is working.\n");
      printf("help:       Displays this message.\n\n");
    }
    return(0);
  }

  if (!File_Exists(values[sw_master]))
  {
    printf("Master file '%s' not found\n", values[sw_master]);
    return(0);
  }

  if (!File_Exists(values[sw_translated]))
  {
    printf("Translated file '%s' not found\n", values[sw_translated]);
    return(0);
  }


  /* Open master file */
  fpm = fopen(values[sw_master], "r");
  if (fpm == NULL)
  {
    printf("Couldn't open file '%s'\n", values[sw_master]);
    return(0);
  }
  else
  {
    if (verbose) printf("\nMaster file '%s' opened...\n", values[sw_master]);
    else printf("\nMaster file opened...\n");

    /* Open translated file for token reading */
    MsgTrans_LoadFile(&msgsptr, values[sw_translated]);
    if (verbose) printf("Translated file '%s' opened...\n", values[sw_translated]);
    else printf("Translated file opened...\n");

    /* Open output file for writing */
    fpo = fopen(values[sw_output], "w");
    if (fpo == NULL)
    {
      fclose(fpm);
      printf(" -> Failed to open file '%s' <-\n", values[sw_output]);
      return(0);
    }
    if (verbose) printf("Output file '%s' opened...\n\n", values[sw_output]);
    else printf("Output file opened...\n\n");

    while(fgets(rawline, LINE_LENGTH, fpm)) /* Read in a line at a time from the master file */
    {
      if ((strncmp(rawline, "#", 1) != 0) && (strlen(rawline) > 1)) /* Skip comments & blank lines */
      {
        strcpy(line, rawline); /* Preserve rawline for possible use later */
        ptr = strtok(line, ":");
        if (ptr != NULL)
        {
          strncpy(token, ptr, TOKEN_LENGTH);  /* Get token */
          token[TOKEN_LENGTH-1]='\0'; /* Ensure termination */
        }
        outputvalue[0] = '\0'; /* Ensure outputvalue is not the same as last time round the loop */
        if (verbose) printf("    Looking for token '%s'... ",token);
        error = MsgTrans_Lookup(msgsptr, token, outputvalue, VALUE_LENGTH);
        if ((error == NULL) && (strlen(outputvalue) != 0))
        { /* Token is present in translation */
          if (verbose) printf("found\n");
          fprintf(fpo, "%s:%s\n", token, outputvalue);
        }
        else
        { /* Token missing in translation */
          missingtokens++;
          if (verbose) printf("not found\n");
          printf("--> Translation for token '%s' not found, using value from master file <--\n", token);
          fprintf(fpo, "%s", rawline); /* Use rawline from original file instead (includes a newline) */
        }
      }
    }
    fclose(fpo);
    MessageTrans_CloseFile(msgsptr);
    printf("\nFinished");
    if (missingtokens>0)
    {
      if (missingtokens==1)
        printf(": %d token translation not found\n", missingtokens);
      else
        printf(": %d token translations not found\n", missingtokens);
    }
    else
      printf(": all OK\n");
  }

  return(0);
}

